/*
 * AppTaskUart0.c
 *
 *  Created on: 2022年2月22日
 *      Author: QiXiang_CHENJIE
 */
#include "AppTaskUart0.h"
#ifdef APP_UART0_ENABLE
static process_Uart0 gProcess_Uart0_Task = PROCESS_UART_STATE_IDLE;
#define PROC_UART0_STATE_SWITCH(a) (gProcess_Uart0_Task = a)
bool bmsCellInfoDecode(uint8 *dataPtr);
bool bmsTempInfoDecode(uint8 *dataPtr);
bool bmsOtherInfoDecode(uint8 *dataPtr);
void UartBusReadDataFunc(uint16 RegAddrBegin, uint16 Reg_Num, bool (*decodeFunc)(uint8 *dataPtr));
// void battDataCalFunc(void);
static uint16 UartNoDataReadCounter = 0;
void Uart0Task(void *pvParameters)
{
	(void)pvParameters;
	UartAnsType UartAnsData;
	uint16 pReadLen = 0;
	static uint8 writecounter = 0;
	while (1)
	{
		switch (gProcess_Uart0_Task)
		{
		case PROCESS_UART_STATE_IDLE:
		{
			vTaskDelay(pdMS_TO_TICKS(10));
			if ((TimerCounter % 1000) == 0 && 1)
			{
				PROC_UART0_STATE_SWITCH(PROCESS_UART_STATE_READ);
			}
			else if (TimerCounter % 100 == 0)
			{
				PROC_UART0_STATE_SWITCH(PROCESS_UART_STATE_WRITE);
			}
			break;
		}
		case PROCESS_UART_STATE_READ:
		{
			Dio_FlipChannel(DioConf_DioChannel_PTE1_GPIO_OUT_MCU_LED2);
			//其他参数读取
			UartBusReadDataFunc(0x01, 0x38, bmsOtherInfoDecode);
			//电压读取
			UartBusReadDataFunc(0x64, 120, bmsCellInfoDecode);
			//温度读取
			UartBusReadDataFunc(0x190, 40, bmsTempInfoDecode);
			//没有读到数据时的默认值
			if (UartNoDataReadCounter > 10)
			{
				memset((uint8 *)&UartAnsData, 0x00, sizeof(UartAnsData));
				bmsCellInfoDecode(UartAnsData.Data);
				bmsTempInfoDecode(UartAnsData.Data);
				bmsOtherInfoDecode(UartAnsData.Data);
				PutErrorNum(ErrorArray, sizeof(ErrorArray) / 2, 1);
				vTaskDelay(pdMS_TO_TICKS(5000));
			}
			PROC_UART0_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
			break;
		}
		case PROCESS_UART_STATE_WRITE:
		{
			if (battSeparateEnable) //最多重新写5次
			{
				uint16 RegAddress = 0;
				uint16 CRC_chk_buffer = 0;
				uint8 WriteData[2] = {0x00, 0x00};
				uint8 UartRecvBuffer[10];
				if (!battSeparateCtlState)
				{
					WriteData[1] = 0x55;
				}
				else
				{
					WriteData[1] = 0xAA;
				}
				RegAddress = 100;
				UartWriteMsgType Uart_Write_Msg;
				Uart_Write_Msg.Bms_Address = BMS_ADDRESS_CODE;
				Uart_Write_Msg.Bms_Funcode = UART_WRITE_CODE;
				Uart_Write_Msg.Reg_Begin_H = RegAddress >> 8;
				Uart_Write_Msg.Reg_Begin_L = RegAddress;
				Uart_Write_Msg.Reg_Num_H = 0x00;
				Uart_Write_Msg.Reg_Num_L = 0x01;
				Uart_Write_Msg.Data_Count = 0x02; //要写入的字节数
				memcpy(Uart_Write_Msg.Data, WriteData, 2);
				CRC_chk_buffer = crc_chk((uint8 *)&Uart_Write_Msg, sizeof(Uart_Write_Msg) - 2);
				Uart_Write_Msg.CRC_L = CRC_chk_buffer;
				Uart_Write_Msg.CRC_H = CRC_chk_buffer >> 8;
				UART_Query_Data(UART_LPUART0, UART_LPUART0, (uint8 *)&Uart_Write_Msg, sizeof(Uart_Write_Msg), UartRecvBuffer, &pReadLen, 500);
				if (pReadLen > 3 && *(UartRecvBuffer + 1) == 0x10)
				{
					writecounter = 0;
					battSeparateEnable = 0;
				}
				else
				{
					writecounter++;
				}
			}
			if (writecounter >= 5)
			{
				battSeparateEnable = 0;
				writecounter = 0;
			}
			PROC_UART0_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
			break;
		}
		case PROCESS_UART_STATE_SLEEP:
		{
			break;
		}
		default:
			break;
		}
	}
}
bool bmsCellInfoDecode(uint8 *dataPtr) //电压数据解码
{
	for (uint8 i = 0; i < BMS_CELLNUM; i++)
	{
		battCellU[i] = ((dataPtr[(0x00 + i) * 2] << 8) | dataPtr[(0x00 + i) * 2 + 1]);
	}
	return true;
}
bool bmsTempInfoDecode(uint8 *dataPtr) //温度数据解码
{
	for (uint8 i = 0; i < BMS_TEMPNUM; i++)
	{
		battCellTemp[i] = dataPtr[(0x00 + i) * 2 + 1];
	}
	return true;
}
bool bmsOtherInfoDecode(uint8 *dataPtr) //其他数据解析
{
	uint16 Batt_current;
	battPackVol = (dataPtr[(0x00) * 2]) << 8 | (dataPtr[(0x00) * 2 + 1]);
	Batt_current = (dataPtr[(0x01) * 2] << 8) | (dataPtr[(0x01) * 2 + 1]);
	battI = Batt_current - 6000; //电流偏移量不同
	battSOC = dataPtr[(0x02) * 2 + 1];
	battSOH = dataPtr[(0x03) * 2 + 1];
	battWorkState = (dataPtr[(0x06) * 2 + 1]) & 0x03; //电池状态(原始数据)，3表示开路，2表示放电，1表示充电
	switch (battWorkState)							  //电池状态(上传数据)，0表示开路，1表示放电，2表示充电
	{
	case 0:
		battWorkState = 0;
		break;
	case 1:
		battWorkState = 2;
		break;
	case 2:
		battWorkState = 1;
		break;
	case 3:
		battWorkState = 0;
		break;
	default:
		break;
	}
	maxCellVol = (dataPtr[(0x13) * 2] << 8) | dataPtr[(0x13) * 2 + 1];
	minCellVol = (dataPtr[(0x16) * 2] << 8) | dataPtr[(0x16) * 2 + 1];
	AppConfigInfo.AppDataInfo.battDischrgAccEnrg = (dataPtr[(0x34) * 2] << 8) | dataPtr[(0x34) * 2 + 1];
	AppConfigInfo.AppDataInfo.battCycleTimes = AppConfigInfo.AppDataInfo.battDischrgAccEnrg / 49000;
	// //故障状态映关系(先完成告警状态简单上传)
	// uint8 ErrorArrayTemp[32] = {0};
	// uint8 ErrorArrayNumTemp[32] = {81, 87, 83, 89, 97, 99, 132, 131, 134, 84, 90, 82, 88, 135, 136, 98, 100};
	// ErrorArrayTemp[0] = getbit(dataPtr[(0x03 + OtherInfoAddrOffset) * 2], 0);				  //电芯电压过高切断告警-0-81
	// ErrorArrayTemp[1] = getbit(dataPtr[(0x03 + OtherInfoAddrOffset) * 2], 1);				  //总压过高切断告警-1-87
	// ErrorArrayTemp[2] = getbit(dataPtr[(0x04 + OtherInfoAddrOffset) * 2 + 1], 0);			  //电芯电压过低切断告警2-83
	// ErrorArrayTemp[3] = getbit(dataPtr[(0x04 + OtherInfoAddrOffset) * 2 + 1], 1);			  //总压过低切断告警3-89
	// ErrorArrayTemp[4] = getbit(dataPtr[(0x04 + OtherInfoAddrOffset) * 2], 4);				  //单节温度过高切断4-97
	// ErrorArrayTemp[5] = getbit(dataPtr[(0x04 + OtherInfoAddrOffset) * 2], 5);				  //单节温度过低切断5-99
	// ErrorArrayTemp[6] = getbit(dataPtr[(0x05 + OtherInfoAddrOffset) * 2 + 1], 1);			  //放电电流切断6-132
	// ErrorArrayTemp[7] = getbit(dataPtr[(0x05 + OtherInfoAddrOffset) * 2 + 1], 2);			  //充电电流切断7-131
	// ErrorArrayTemp[8] = getbit(dataPtr[(0x05 + OtherInfoAddrOffset) * 2 + 1], 3);			  // BTU连接告警8-134
	// ErrorArrayTemp[9] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 0);  //单节电压下限告警9-84
	// ErrorArrayTemp[10] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 1); //组电压下限告警10-90
	// ErrorArrayTemp[11] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 2); //单节电压上限告警11-82
	// ErrorArrayTemp[12] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 3); //组单节电压上限告警12-88
	// ErrorArrayTemp[13] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 4); //放电电流告警13-135
	// ErrorArrayTemp[14] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 5); //充电电流告警14-136
	// ErrorArrayTemp[15] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 6); //单节温度上限告警15-98
	// ErrorArrayTemp[16] = getbit(dataPtr[(0x0A + OtherInfoAddrOffset + TEMP_NUM) * 2 + 1], 7); //单节温度下线告警16-100
	// for (uint8 i = 0, j = 0; (i < 32 && j < 5); i++)
	// {
	// 	if (ErrorArrayTemp[i] == 1)
	// 	{
	// 		PutErrorNum(ErrorArray, sizeof(ErrorArray) / 2, ErrorArrayNumTemp[i]);
	// 		j++;
	// 	}
	// }
	return true;
}
// void battDataCalFunc(void)
// {
// 	static uint32 timercounterCal = 0;
// 	uint16 timediff = 0;
// 	float battCapTempOnce = 0;
// 	static float battCapTemp = 0;
// 	static float battEnrgTemp = 0;
// 	timediff = TimerCounter - timercounterCal;
// 	timercounterCal = TimerCounter;
// 	if (timediff == 0 || timediff > 60 * 1000)
// 	{
// 		return;
// 	}
// 	if (battI > 10000)
// 	{
// 		battCapTempOnce = ((battI - 10000) / 10.0) * (timediff / 3600.0);
// 		battCapTemp = battCapTempOnce + battCapTemp;
// 		battEnrgTemp = (battCapTempOnce * battPackVol) / 10000.0 + battEnrgTemp;
// 		if (battEnrgTemp > 10)
// 		{
// 			AppConfigInfo.AppDataInfo.battDischrgAccCap = AppConfigInfo.AppDataInfo.battDischrgAccCap + (uint32)battCapTemp;
// 			AppConfigInfo.AppDataInfo.battDischrgAccEnrg = AppConfigInfo.AppDataInfo.battDischrgAccEnrg + (uint32)battEnrgTemp;
// 			AppConfigInfo.AppDataInfo.battCycleTimes = AppConfigInfo.AppDataInfo.battDischrgAccEnrg / 49000;
// 			AppConfigInfo.appSaveFlg = true;
// 			battEnrgTemp = 0;
// 			battCapTemp = 0;
// 		}
// 	}
// }

void UartBusReadDataFunc(uint16 RegAddrBegin, uint16 Reg_Num, bool (*decodeFunc)(uint8 *dataPtr))
{
	// argv In
	UartQueryType Uart0AskMsg;
	uint16 ReadLen1 = 0;
	uint16 ReadLenTar = 0;
	uint8 ReadDelayCounter = 0;
	uint16 CRC_chk_buffer = 0;
	uint16 pReadLen;
	UartAnsType UartAnsData;
	uint8 UartDataRecv[256];
	pReadLen = 0;
	Uart0AskMsg.Bms_Address = BMS_ADDRESS_CODE;
	Uart0AskMsg.Bms_Funcode = UART_READ_CODE;
	Uart0AskMsg.Reg_Begin_H = RegAddrBegin >> 8;
	Uart0AskMsg.Reg_Begin_L = RegAddrBegin;
	Uart0AskMsg.Reg_Num_H = Reg_Num >> 8;
	Uart0AskMsg.Reg_Num_L = Reg_Num;
	CRC_chk_buffer = crc_chk((uint8 *)&Uart0AskMsg, 6);
	Uart0AskMsg.CRC_L = CRC_chk_buffer;
	Uart0AskMsg.CRC_H = CRC_chk_buffer >> 8;
	UART_Send_Data(UART_LPUART0, (uint8 *)&Uart0AskMsg, sizeof(Uart0AskMsg), 1000);
	ReadLenTar = (Reg_Num)*2 + 5;
	memset((uint8 *)&UartAnsData, 0x00, sizeof(UartAnsData));
	while (1)
	{
		UART_Receive_Data(UART_LPUART0, UartDataRecv, &pReadLen, 100);
		if (pReadLen > 2)
		{
			memcpy((uint8 *)&UartAnsData + ReadLen1, UartDataRecv, pReadLen);
			ReadLen1 = ReadLen1 + pReadLen;
		}
		else
		{
			ReadDelayCounter++;
		}
		if (ReadLen1 >= ReadLenTar)
		{
			ReadDelayCounter = 0;
			pReadLen = ReadLen1;
			ReadLen1 = 0;
			break;
		}
		if (ReadDelayCounter >= 20)
		{
			ReadDelayCounter = 0;
			pReadLen = 0;
			ReadLen1 = 0;
			break;
		}
	}
	if (pReadLen > 0)
	{
		uint16 CrcChkGet = 0xffff;
		uint16 CrcChkCal = 0x0000;
		CrcChkCal = crc_chk((uint8 *)&UartAnsData, pReadLen - 2);
		CrcChkGet = ((uint16)(UartAnsData.Data[pReadLen - 1 - 3]) << 8) | ((uint16)(UartAnsData.Data[pReadLen - 2 - 3]));
		if (CrcChkCal == CrcChkGet)
		{
			decodeFunc(UartAnsData.Data);
			UartNoDataReadCounter = 0;
		}
		else
		{
			UartNoDataReadCounter++;
		}
	}
	else
	{
		UartNoDataReadCounter++;
	}
}

#endif
